--[[local version = GetBuildInfo()
if not version:match("3.3.5") then
	print("HideBlizzard will not work until your game version is updated to 3.3.5")
	return
end
]]
local HideBlizzard = LibStub("AceAddon-3.0"):NewAddon("HideBlizzard", "AceConsole-3.0", "AceEvent-3.0")
assert(HideBlizzard, "Core::HideBlizzard does not exist")

local _VERSION = GetAddOnMetadata("HideBlizzard", "Version")
local wait = CreateFrame("Frame")
local wipe, type = wipe, type

local defaults = {
	profile = {
		modules = {
			["ActionBar"] = nil,
			["Buttons"] = nil,
			["Pet"] = nil,
			["Player"] = nil,
			["SpecialFrames"] = nil,
			["Target"] = nil,
			["Vehicle"] = nil,
		},
	},
}

HideBlizzard:SetEnabledState(true)
HideBlizzard:SetDefaultModuleState(false)
HideBlizzard:SetDefaultModuleLibraries("AceEvent-3.0")

function HideBlizzard:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HideBlizzardDB", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateView")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateView")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateView")
	db = self.db.profile

	self:RegisterOptions()
	self:RegisterChatCommand("hb", "RegisterCommands")
	self:RegisterChatCommand("hideblizzard", "RegisterCommands")
end

function HideBlizzard:OnEnable()
	local lastUpdate = 0
	wait:SetScript("OnUpdate", function(self, elapsed)
		lastUpdate = lastUpdate + elapsed
		if (lastUpdate > 3) then
			if not db.Welcome and not db.Version then
				db.Welcome = true
				db.Version = _VERSION
				HideBlizzard:ShowWelcome()
--				HideBlizzard:Print("Welcome to |cff40bbffHideBlizzard!|r Type |cffffff33/hb|r to see the commands")
			elseif db.Welcome and db.Version ~= _VERSION then
				if db.Version:match("1.0.55") then
					HideBlizzardDB = {}
					HideBlizzard:ShowUpgrade()
					lastUpdate = 0
					wait:SetScript("OnUpdate", nil)
				else
					db.Version = _VERSION
					HideBlizzard:ShowChangelog()
					HideBlizzard:Print(string.format("Updated to version |cff00ff66%s|r.", db.Version))
				end
			end
			lastUpdate = 0
			wait:SetScript("OnUpdate", nil)
		end
	end)

	self:RegisterEvent("PLAYER_REGEN_DISABLED", function()
		if InterfaceOptionsFrame:IsVisible() then
			self:Print("|cff40bbffHideBlizzard|r is closing the options window since you are currently in combat")
			InterfaceOptionsFrame:Hide()
			GameMenu:Hide()
		end
	end)
end

function HideBlizzard:OnDisable()
end

function HideBlizzard:UpdateView()
	for k,v in self:IterateModules() do
		if self:GetModuleEnabled(k) and not v:IsEnabled() then
			self:EnableModule(k)
		elseif not self:GetModuleEnabled(k) and v:IsEnabled() then
			self:DisableModule(k)
		end
		if type(v.UpdateView) == "function" then
			v:UpdateView()
		end
	end
	FramePositionDelegate:SetAttribute("uiparent-manage", true)
--	print("updating interface..")
end

function HideBlizzard:GetModuleEnabled(module)
	return self.db.profile.modules[module]
end

function HideBlizzard:SetModuleEnabled(module, value)
	local old = HideBlizzard.db.profile.modules[module]
	HideBlizzard.db.profile.modules[module] = value
	if old ~= value then
		if value then
			HideBlizzard:EnableModule(module)
		else
			HideBlizzard:DisableModule(module)
		end
	end
end

function HideBlizzard:Reset()
	wipe(HideBlizzardDB)
	self:Print("Database reset, will take effect when you next login")
end

function HideBlizzard:Open()
	InterfaceOptionsFrame_OpenToCategory(HideBlizzard.optionFrame.Profiles)
	InterfaceOptionsFrame_OpenToCategory(HideBlizzard.optionFrame.HideBlizzard)
	InterfaceOptionsFrame:Raise()
end

function HideBlizzard:RegisterCommands(args)
	local cmd = string.split(" ", args):lower() or args:lower()
	local function PrintCmd(cmd)
		DEFAULT_CHAT_FRAME:AddMessage(string.format("|cffffff33%s|r", cmd))
	end
	local function PrintHelp(cmd, desc)
		DEFAULT_CHAT_FRAME:AddMessage(string.format("|cffffff33  %s|r - %s", cmd, desc))
	end

	if (cmd == "changelog") then
		self:ShowChangelog()
	elseif (cmd == "config") then
		self:Open()
	elseif (cmd == "help") then
		self:ShowHelp()
	elseif (cmd == "reset") then
		self:Reset()
	elseif (cmd == "version") then
		self:Print(string.format("Your current version is |cff00ff66%s|r", db.Version))
	else
		PrintCmd("|cff00ff66Available Commands for|r |cff40bbffHideBlizzard (/hb)|r", "\n")
		PrintHelp("changelog", "view the recent changes")
		PrintHelp("config", "opens the option window")
		PrintHelp("help", "open the help dialog")
		PrintHelp("reset", "resets the HideBlizzard database")
		PrintHelp("version", "prints the current version")
	end
end